/*****************************************************************************

  The following code is derived, directly or indirectly, from the SystemC
  source code Copyright (c) 1996-2014 by all Contributors.
  All Rights reserved.

  The contents of this file are subject to the restrictions and limitations
  set forth in the SystemC Open Source License (the "License");
  You may not use this file except in compliance with such restrictions and
  limitations. You may obtain instructions on how to receive a copy of the
  License at http://www.accellera.org/. Software distributed by Contributors
  under the License is distributed on an "AS IS" basis, WITHOUT WARRANTY OF
  ANY KIND, either express or implied. See the License for the specific
  language governing rights and limitations under the License.

 *****************************************************************************/

// Copyright 2018, Xilinx Inc.  All rights reserved.
// Following changes have been made by Xilinx:
//     Placed under the xilinx_sc namespace 
//     Changes made to use Xilinx Simulation Scheduler

/*****************************************************************************

  sc_main.cpp - Wrapper around user's toplevel routine `sc_main'.

  Original Author: Stan Y. Liao, Synopsys, Inc.

 CHANGE LOG APPEARS AT THE END OF THE FILE
 *****************************************************************************/

#if defined(_WIN32)
    #define XILINX_SYSTEMC_DLL __declspec(dllexport)
#endif



#include "sysc/kernel/sc_cmnhdr.h"
#include "sysc/kernel/sc_externs.h"
#include "sysc/kernel/sc_xilinx_cor.h"
#include "sysc/kernel/sc_simcontext.h"

// Includes from the HDL side
// DO not include any XSIM kernel header other than iki as this code needs
// to be compiled using MinGW on Windows
#include "iki.h"

#include "systemcmgr.h"

#ifdef _WIN32
#define WEAK_LINKAGE
#else
#define WEAK_LINKAGE __attribute__((weak))
#endif


extern "C" {
    SYSTEMCLIB_EXP_DLLSPEC void local_register_implicit_channel(int channelNumber, char* channel);
    SYSTEMCLIB_EXP_DLLSPEC void local_write_channel_int(int channelNumber, char* newValue, int numBytes);
    SYSTEMCLIB_EXP_DLLSPEC void local_sc_set_curr_proc(void* proc);
    SYSTEMCLIB_EXP_DLLSPEC void local_xsim_sc_init_elab();
    SYSTEMCLIB_EXP_DLLSPEC void local_reset();
    SYSTEMCLIB_EXP_DLLSPEC int local_sc_pending_activity_at_future_time();
    SYSTEMCLIB_EXP_DLLSPEC int local_pending_updates();
    SYSTEMCLIB_EXP_DLLSPEC void local_set_evaluate_phase();
    SYSTEMCLIB_EXP_DLLSPEC int local_run_update_and_notification(int empty_eval_phase);
    SYSTEMCLIB_EXP_DLLSPEC void local_trigger_timed_event(int* event);
    SYSTEMCLIB_EXP_DLLSPEC int local_check_trace_pause_error();
    SYSTEMCLIB_EXP_DLLSPEC void local_do_sc_stop_action();
    SYSTEMCLIB_EXP_DLLSPEC int local_is_sc_forced_stop();
    SYSTEMCLIB_EXP_DLLSPEC int local_is_sc_stop_immediate();
    SYSTEMCLIB_EXP_DLLSPEC const char* local_do_sc_report(char* x);
    SYSTEMCLIB_EXP_DLLSPEC char* local_sc_get_curr_simcontext();
    SYSTEMCLIB_EXP_DLLSPEC void local_setSCSimContext(char* scSimContext);
    SYSTEMCLIB_EXP_DLLSPEC char* local_getSCSimContext();
    SYSTEMCLIB_EXP_DLLSPEC int local_is_sc_thread_proc(char* scSimContext);
    SYSTEMCLIB_EXP_DLLSPEC void local_sc_process_timed_notifications();
    SYSTEMCLIB_EXP_DLLSPEC void local_update_change_stamp();
}


void local_register_implicit_channel(int channelNumber, char* channel)
{
    xilinx_sc::sc_core::sc_get_curr_simcontext()->register_implicit_channel(channelNumber, channel);
}

void local_write_channel_int(int channelNumber, char* newValue, int numBytes)
{
    xilinx_sc::sc_core::sc_get_curr_simcontext()->write_channel_int(channelNumber, newValue, numBytes);
}

void local_sc_set_curr_proc(void* proc)
{
    xilinx_sc::sc_core::sc_get_curr_simcontext()->bridge_set_curr_proc(proc);
}

void local_xsim_sc_init_elab()
{
    xilinx_sc::sc_core::sc_get_curr_simcontext()->xsim_sc_init_elab();
}

void local_reset()
{
    xilinx_sc::sc_core::sc_get_curr_simcontext()->reset();
}

int local_sc_pending_activity_at_future_time()
{
    return xilinx_sc::sc_core::sc_get_curr_simcontext()->sc_pending_activity_at_future_time();
}

int local_pending_updates()
{
    return xilinx_sc::sc_core::sc_get_curr_simcontext()->pending_updates();
}

void local_set_evaluate_phase()
{
    xilinx_sc::sc_core::sc_get_curr_simcontext()->set_evaluate_phase();
}

int local_run_update_and_notification(int empty_eval_phase)
{
    return xilinx_sc::sc_core::sc_get_curr_simcontext()->run_update_and_notification(empty_eval_phase);
}

void local_trigger_timed_event(int* event)
{
    xilinx_sc::sc_core::sc_get_curr_simcontext()->trigger_timed_event(event);
}

int local_check_trace_pause_error()
{
    return xilinx_sc::sc_core::sc_get_curr_simcontext()->check_trace_pause_error();
}

void local_do_sc_stop_action()
{
    xilinx_sc::sc_core::sc_get_curr_simcontext()->do_sc_stop_action();
}

int local_is_sc_forced_stop()
{
    return xilinx_sc::sc_core::sc_get_curr_simcontext()->is_sc_forced_stop();
}

int local_is_sc_stop_immediate()
{
    return xilinx_sc::sc_core::sc_get_curr_simcontext()->is_sc_stop_immediate();
}

const char* local_do_sc_report(char* x)
{
    return xilinx_sc::sc_core::sc_get_curr_simcontext()->do_sc_report(x);
}

char* local_sc_get_curr_simcontext()
{
    return (char*)(xilinx_sc::sc_core::sc_get_curr_simcontext());
}

void local_setSCSimContext(char* scSimContext)
{
    void* pSystemCMgr = iki_get_systemcmgr();
    // ((SystemCMgr*)pSystemCMgr)->setSCSimContext(scSimContext);
    ((SystemCMgr*)pSystemCMgr)->m_SCSimContext = scSimContext;
}

char* local_getSCSimContext()
{
    void* pSystemCMgr = iki_get_systemcmgr();
    // return (char*)(((SystemCMgr*)pSystemCMgr)->getSCSimContext());
    return (char*)(((SystemCMgr*)pSystemCMgr)->m_SCSimContext);
}

int local_is_sc_thread_proc(char* scSimContext)
{
    return xilinx_sc::sc_core::sc_get_curr_simcontext()->is_sc_thread_proc(scSimContext);
}

void local_sc_process_timed_notifications()
{
    xilinx_sc::sc_core::sc_get_curr_simcontext()->sc_process_timed_notifications();
}


void local_update_change_stamp()
{
    xilinx_sc::sc_core::sc_get_curr_simcontext()->update_change_stamp();
}

// extern "C" int WEAK_LINKAGE main( int argc, char* argv[] )
int WEAK_LINKAGE main( int argc, char* argv[] )
{
    int result = 0;

/*
    void* pSystemCMgr = iki_get_systemcmgr();
    ((SystemCMgr*)pSystemCMgr)->programFunctionPointers(
*/
    iki_sc_programFunctionPointers(
        local_register_implicit_channel,
        local_write_channel_int, 
        local_sc_pending_activity_at_future_time, 
        local_xsim_sc_init_elab,
        local_reset,
        local_sc_set_curr_proc,
        local_sc_get_curr_simcontext,
        local_pending_updates,
        local_set_evaluate_phase,
        local_run_update_and_notification,
        local_trigger_timed_event,
        local_check_trace_pause_error,
        local_do_sc_stop_action,
        local_is_sc_forced_stop,
        local_is_sc_stop_immediate,
        local_do_sc_report,
        local_setSCSimContext,
        local_getSCSimContext,
        local_is_sc_thread_proc,
        local_sc_process_timed_notifications,
        local_update_change_stamp);

    void* design_handle = iki_sc_create_design(argc, (char**) argv);

    (void) design_handle;

    // coroutine package will be created in xsim_sc_elaborate()
    // Create the coroutine package that we will use for THREAD yielding
    //sc_core::globalCoroutinePackage = new sc_core::sc_xilinx_cor_pkg() ;

    // Perform the SystemC execution
    result = xilinx_sc::sc_core::sc_elab_and_sim(argc, argv);

    // Clean up our global structures
    delete xilinx_sc::sc_core::globalCoroutinePackage;

    // The XSIM design handle will be cleaned up when program exits, no need to destruct them
    // If needed to clean up explicitly fo some reason, then add an iki_clean_design(void* design_handle)

    return result;
}

#ifdef _WIN32
#include <windows.h>
#include <malloc.h>	

/* Parse a command line buffer into arguments */
static int ParseCommandLine(char *cmdline, char **argv)
{
 	char *bufp;
 	int argc;
 
 	argc = 0;
	for ( bufp = cmdline; *bufp; ) {
 		/* Skip leading whitespace */
 		while ( isspace(*bufp) ) {
 			++bufp;
 		}
		/* Skip over argument */
 		if ( *bufp == '"' ) {
			++bufp;
 			if ( *bufp ) {
 				if ( argv ) {
 					argv[argc] = bufp;
				}
				++argc;
 			}
 			/* Skip over word */
 			while ( *bufp && (*bufp != '"') ) {
 				++bufp;
 			}
 		} else {
 			if ( *bufp ) {
 				if ( argv ) {
					argv[argc] = bufp;
 				}
				++argc;
 			}
 			/* Skip over word */
			while ( *bufp && ! isspace(*bufp) ) {
 				++bufp;
 			}
 		}
 		if ( *bufp ) {
			if ( argv ) {
 				*bufp = '\0';
			}
 			++bufp;
		}
 	}
	if ( argv ) {
		argv[argc] = NULL;
 	}
 	return(argc);
}

extern "C" XILINX_SYSTEMC_DLL WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrev, LPSTR szCmdLine, int sw)
{
 	char **argv;
 	int argc;
 	char *cmdline;
 	char *bufp;
 
 	/* Grab the command line (use alloca() on Windows) */
 	bufp = GetCommandLine();
 	cmdline = (char *)alloca(strlen(bufp)+1);
 	strcpy(cmdline, bufp);
 
 	/* Parse it into argv and argc */
 	argc = ParseCommandLine(cmdline, NULL);
 	argv = (char **)alloca((argc+1)*(sizeof *argv));
 	ParseCommandLine(cmdline, argv);
 
 	/* Run the main program  */
 	return(main(argc, argv));
}
#endif

// $Log: sc_main.cpp,v $
// Revision 1.5  2011/08/26 20:46:09  acg
//  Andy Goodrich: moved the modification log to the end of the file to
//  eliminate source line number skew when check-ins are done.
//
// Revision 1.4  2011/02/18 20:27:14  acg
//  Andy Goodrich: Updated Copyrights.
//
// Revision 1.3  2011/02/13 21:47:37  acg
//  Andy Goodrich: update copyright notice.
//
// Revision 1.2  2008/05/22 17:06:25  acg
//  Andy Goodrich: updated copyright notice to include 2008.
//
// Revision 1.1.1.1  2006/12/15 20:20:05  acg
// SystemC 2.3
//
// Revision 1.3  2006/01/13 18:44:29  acg
// Added $Log to record CVS changes into the source.
//

// 67d7842dbbe25473c3c32b93c0da8047785f30d78e8a024de1b57352245f9689
